using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class ConfigController : ControllerBase
    {
        private readonly ILogger<ConfigController> _logger;
        private readonly IConfigService _configService;

        public ConfigController(
            ILogger<ConfigController> logger,
            IConfigService configService)
        {
            _logger = logger;
            _configService = configService;
        }

        [HttpGet]
        public IActionResult GetConfig()
        {
            var config = _configService.GetConfiguration();
            
            // Create a sanitized config without sensitive information
            var sanitizedConfig = new
            {
                transmission = new
                {
                    host = config.Transmission.Host,
                    port = config.Transmission.Port,
                    useHttps = config.Transmission.UseHttps,
                    hasCredentials = !string.IsNullOrEmpty(config.Transmission.Username),
                    username = config.Transmission.Username
                },
                transmissionInstances = config.TransmissionInstances?.Select(i => new
                {
                    id = i.Key,
                    name = i.Value.Host,
                    host = i.Value.Host,
                    port = i.Value.Port,
                    useHttps = i.Value.UseHttps,
                    hasCredentials = !string.IsNullOrEmpty(i.Value.Username),
                    username = i.Value.Username
                }),
                autoDownloadEnabled = config.AutoDownloadEnabled,
                checkIntervalMinutes = config.CheckIntervalMinutes,
                downloadDirectory = config.DownloadDirectory,
                mediaLibraryPath = config.MediaLibraryPath,
                postProcessing = config.PostProcessing,
                enableDetailedLogging = config.EnableDetailedLogging,
                userPreferences = config.UserPreferences
            };
            
            return Ok(sanitizedConfig);
        }
        
        [HttpGet("defaults")]
        public IActionResult GetDefaultConfig()
        {
            // Return default configuration settings
            var defaultConfig = new
            {
                transmission = new
                {
                    host = "localhost",
                    port = 9091,
                    username = "",
                    useHttps = false
                },
                autoDownloadEnabled = true,
                checkIntervalMinutes = 30,
                downloadDirectory = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Downloads"),
                mediaLibraryPath = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Media"),
                postProcessing = new
                {
                    enabled = false,
                    extractArchives = true,
                    organizeMedia = true,
                    minimumSeedRatio = 1,
                    mediaExtensions = new[] { ".mp4", ".mkv", ".avi" },
                    autoOrganizeByMediaType = true,
                    renameFiles = false,
                    compressCompletedFiles = false,
                    deleteCompletedAfterDays = 0
                },
                enableDetailedLogging = false,
                userPreferences = new
                {
                    enableDarkMode = false,
                    autoRefreshUIEnabled = true,
                    autoRefreshIntervalSeconds = 30,
                    notificationsEnabled = true,
                    notificationEvents = new[] { "torrent-added", "torrent-completed", "torrent-error" },
                    defaultView = "dashboard",
                    confirmBeforeDelete = true,
                    maxItemsPerPage = 25,
                    dateTimeFormat = "yyyy-MM-dd HH:mm:ss",
                    showCompletedTorrents = true,
                    keepHistoryDays = 30
                }
            };
            
            return Ok(defaultConfig);
        }

        [HttpPut]
        public async Task<IActionResult> UpdateConfig([FromBody] AppConfig config)
        {
            var currentConfig = _configService.GetConfiguration();
            
            // If password is empty, keep the existing one
            if (string.IsNullOrEmpty(config.Transmission.Password) && !string.IsNullOrEmpty(currentConfig.Transmission.Password))
            {
                config.Transmission.Password = currentConfig.Transmission.Password;
            }
            
            await _configService.SaveConfigurationAsync(config);
            return Ok(new { success = true });
        }
        
        [HttpPost("backup")]
        public IActionResult BackupConfig()
        {
            try
            {
                // Get the current config
                var config = _configService.GetConfiguration();
                
                // Serialize to JSON with indentation
                var options = new JsonSerializerOptions { WriteIndented = true };
                var json = JsonSerializer.Serialize(config, options);
                
                // Create a memory stream from the JSON
                var stream = new MemoryStream(Encoding.UTF8.GetBytes(json));
                
                // Set the content disposition and type
                var fileName = $"transmission-rss-config-backup-{DateTime.Now:yyyy-MM-dd}.json";
                return File(stream, "application/json", fileName);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error creating configuration backup");
                return StatusCode(500, "Error creating configuration backup");
            }
        }
        
        [HttpPost("reset")]
        public async Task<IActionResult> ResetConfig()
        {
            try
            {
                // Create a default config
                var defaultConfig = new AppConfig
                {
                    Transmission = new TransmissionConfig
                    {
                        Host = "localhost",
                        Port = 9091,
                        Username = "",
                        Password = "",
                        UseHttps = false
                    },
                    AutoDownloadEnabled = true,
                    CheckIntervalMinutes = 30,
                    DownloadDirectory = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Downloads"),
                    MediaLibraryPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Media"),
                    PostProcessing = new PostProcessingConfig
                    {
                        Enabled = false,
                        ExtractArchives = true,
                        OrganizeMedia = true,
                        MinimumSeedRatio = 1.0f,
                        MediaExtensions = new[] { ".mp4", ".mkv", ".avi", ".mov", ".wmv", ".m4v", ".mpg", ".mpeg", ".flv", ".webm" },
                        AutoOrganizeByMediaType = true,
                        RenameFiles = false,
                        CompressCompletedFiles = false,
                        DeleteCompletedAfterDays = 0
                    },
                    UserPreferences = new UserPreferencesConfig
                    {
                        EnableDarkMode = true,
                        AutoRefreshUIEnabled = true,
                        AutoRefreshIntervalSeconds = 30,
                        NotificationsEnabled = true,
                        NotificationEvents = new[] { "torrent-added", "torrent-completed", "torrent-error" },
                        DefaultView = "dashboard",
                        ConfirmBeforeDelete = true,
                        MaxItemsPerPage = 25,
                        DateTimeFormat = "yyyy-MM-dd HH:mm:ss",
                        ShowCompletedTorrents = true,
                        KeepHistoryDays = 30
                    },
                    Feeds = new List<RssFeed>(),
                    EnableDetailedLogging = false
                };
                
                // Save the default config
                await _configService.SaveConfigurationAsync(defaultConfig);
                
                return Ok(new { success = true });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error resetting configuration");
                return StatusCode(500, "Error resetting configuration");
            }
        }
    }
}